import uuid

from datetime import datetime

from sqlalchemy import DateTime, LargeBinary, Unicode, Uuid
from sqlalchemy.orm import Mapped, mapped_column

from ..db.base_class import reg


@reg.mapped_as_dataclass(unsafe_hash=True)
class DbImage:
    __tablename__ = "images"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    resource_id: Mapped[uuid.UUID] = mapped_column(Uuid, nullable=False)
    resource_type: Mapped[str] = mapped_column(Unicode, nullable=False)
    image: Mapped[bytes] = mapped_column(LargeBinary, nullable=False)
    thumbnail: Mapped[bytes] = mapped_column(LargeBinary, nullable=False)
    creation_date: Mapped[datetime] = mapped_column(DateTime(), nullable=False)

    def __init__(
        self,
        resource_id: uuid.UUID,
        resource_type: str,
        image: bytes,
        thumbnail: bytes,
        creation_date: datetime | None = None,
        id_: uuid.UUID | None = None,
    ) -> None:
        self.resource_id = resource_id
        self.resource_type = resource_type
        self.image = image
        self.thumbnail = thumbnail
        self.creation_date = creation_date or datetime.utcnow()
        if id_ is not None:
            self.id = id_
