import uuid

from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Integer, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .journal import Journal


@reg.mapped_as_dataclass(unsafe_hash=True)
class EmployeeBenefit:
    __tablename__ = "employee_benefit"
    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    voucher_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("vouchers.id"), nullable=False)
    journal_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("journals.id"), nullable=False)
    gross_salary: Mapped[int] = mapped_column(Integer, nullable=False)
    days_worked: Mapped[int] = mapped_column(Integer, nullable=False)
    esi_ee: Mapped[int] = mapped_column("esi_employee", Integer, nullable=False)
    pf_ee: Mapped[int] = mapped_column("pf_employee", Integer, nullable=False)
    esi_er: Mapped[int] = mapped_column("esi_employer", Integer, nullable=False)
    pf_er: Mapped[int] = mapped_column("pf_employer", Integer, nullable=False)

    journal: Mapped["Journal"] = relationship("Journal", back_populates="employee_benefit")

    def __init__(
        self,
        gross_salary: int,
        days_worked: int,
        esi_ee: int,
        pf_ee: int,
        esi_er: int,
        pf_er: int,
        journal: "Journal",
        voucher_id: uuid.UUID | None = None,
        id_: uuid.UUID | None = None,
    ) -> None:
        if id_ is not None:
            self.id = id_
        if voucher_id is not None:
            self.voucher_id = voucher_id
        self.gross_salary = gross_salary
        self.days_worked = days_worked
        self.esi_ee = esi_ee
        self.pf_ee = pf_ee
        self.esi_er = esi_er
        self.pf_er = pf_er
        self.journal = journal
