import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Numeric, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .journal import Journal


@reg.mapped_as_dataclass(unsafe_hash=True)
class Incentive:
    __tablename__ = "incentives"
    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    voucher_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("vouchers.id"), nullable=False)
    journal_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("journals.id"), nullable=False)
    days_worked: Mapped[Decimal] = mapped_column(Numeric(precision=5, scale=1), nullable=False)
    points: Mapped[Decimal] = mapped_column(Numeric(precision=5, scale=2), nullable=False)

    journal: Mapped["Journal"] = relationship("Journal", back_populates="incentive")

    def __init__(
        self,
        journal: "Journal",
        days_worked: Decimal,
        points: Decimal,
        voucher_id: uuid.UUID | None = None,
        id_: uuid.UUID | None = None,
    ) -> None:
        self.journal = journal
        self.days_worked = days_worked
        self.points = points
        if voucher_id is not None:
            self.voucher_id = voucher_id
        if id_ is not None:
            self.id = id_
