import uuid

from decimal import Decimal
from typing import TYPE_CHECKING

from sqlalchemy import ForeignKey, Numeric, UniqueConstraint, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .rate_contract import RateContract
    from .stock_keeping_unit import StockKeepingUnit


@reg.mapped_as_dataclass(unsafe_hash=True)
class RateContractItem:
    __tablename__ = "rate_contract_items"
    __table_args__ = (UniqueConstraint("rate_contract_id", "sku_id"),)

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    rate_contract_id: Mapped[uuid.UUID] = mapped_column(
        Uuid,
        ForeignKey("rate_contracts.id"),
        nullable=False,
        index=True,
    )
    sku_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("stock_keeping_units.id"), nullable=False)
    price: Mapped[Decimal] = mapped_column("cost_price", Numeric(precision=15, scale=2), nullable=False)

    rate_contract: Mapped["RateContract"] = relationship("RateContract", back_populates="items")
    sku: Mapped["StockKeepingUnit"] = relationship("StockKeepingUnit")

    def __init__(
        self,
        rate_contract_id: uuid.UUID,
        sku_id: uuid.UUID,
        price: Decimal,
        id_: uuid.UUID | None = None,
    ) -> None:
        self.rate_contract_id = rate_contract_id
        self.sku_id = sku_id
        self.price = price
        if id_ is not None:
            self.id = id_
