import uuid

from typing import TYPE_CHECKING

from sqlalchemy import Unicode, Uuid
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg
from .role_permission import RolePermission


if TYPE_CHECKING:
    from .permission import Permission


@reg.mapped_as_dataclass(unsafe_hash=True)
class Role:
    __tablename__ = "roles"

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, insert_default=uuid.uuid4)
    name: Mapped[str] = mapped_column(Unicode, unique=True)

    permissions: Mapped[list["Permission"]] = relationship(
        "Permission",
        secondary=RolePermission.__table__,  # type: ignore[attr-defined]
        back_populates="roles",
    )

    def __init__(self, name: str, id_: uuid.UUID | None = None) -> None:
        self.name = name
        if id_ is not None:
            self.id = id_
