from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import Daf, to_camel


class BalanceSheetItem(BaseModel):
    name: str | None = None
    group: str | None = None
    amount: Daf | None = None
    sub_amount: Daf | None = None
    order: int
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class BalanceSheet(BaseModel):
    date_: date | str
    body: list[BalanceSheetItem]
    footer: BalanceSheetItem | None = None
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")
