from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from ..models.voucher_type import VoucherType
from . import to_camel
from .account import AccountLink
from .cost_centre import CostCentreLink


class BlankVoucherInfo(BaseModel):
    type_: VoucherType
    date_: date
    account: AccountLink | None = None
    source: CostCentreLink | None = None
    destination: CostCentreLink | None = None
    model_config = ConfigDict(alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")
