from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import Daf, to_camel


class NonContractPurchase(BaseModel):
    date_: date
    vendor: str
    product: str
    url: list[str]
    contract_price: Daf
    purchase_price: Daf
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_start_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")
