from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import Daf, to_camel


class ProfitLossItem(BaseModel):
    group: str | None = None
    name: str | None = None
    amount: Daf | None = None
    total: Daf | None = None
    order: int
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class ProfitLoss(BaseModel):
    start_date: date | str
    finish_date: date | str
    body: list[ProfitLossItem]
    footer: ProfitLossItem
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("start_date", mode="before")
    @classmethod
    def parse_start_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("start_date")
    def serialize_start_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")

    @field_validator("finish_date", mode="before")
    @classmethod
    def parse_finish_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("finish_date")
    def serialize_finish_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")
