import uuid

from datetime import date, datetime

from pydantic import (
    BaseModel,
    ConfigDict,
    FieldSerializationInfo,
    field_serializer,
    field_validator,
)

from . import Daf, to_camel
from .product import ProductLink
from .recipe_item import RecipeItem


class RecipeIn(BaseModel):
    sku: ProductLink
    date_: date
    source: str
    instructions: str
    garnishing: str
    plating: str
    recipe_yield: Daf
    notes: str

    items: list[RecipeItem]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)

    @field_validator("date_", mode="before")
    @classmethod
    def parse_start_date(cls, value: date | str) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @field_serializer("date_")
    def serialize_date(self, value: date, info: FieldSerializationInfo) -> str:
        return value.strftime("%d-%b-%Y")


class Recipe(RecipeIn):
    id_: uuid.UUID
    product_group_id: uuid.UUID | None = None
    units: str | None = None
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)


class RecipeBlank(RecipeIn):
    sku: ProductLink | None = None  # type: ignore[assignment]
    model_config = ConfigDict(str_strip_whitespace=True, alias_generator=to_camel, populate_by_name=True)
