import { AfterViewInit, Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { FormControl, FormGroup, Validators } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';

import { ToasterService } from '../../core/toaster.service';
import { CookieService } from '../../shared/cookie.service';
import { AuthService } from '../auth.service';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css'],
})
export class LoginComponent implements OnInit, AfterViewInit {
  @ViewChild('nameElement', { static: true }) nameElement?: ElementRef;
  form: FormGroup<{
    username: FormControl<string>;
    password: FormControl<string>;
    otp: FormControl<number | null>;
  }>;

  hide: boolean;
  showOtp: boolean;
  clientId = '';
  private returnUrl = '';

  constructor(
    private route: ActivatedRoute,
    private auth: AuthService,
    private router: Router,
    private toaster: ToasterService,
    private cs: CookieService,
  ) {
    this.hide = true;
    this.showOtp = false;
    this.form = new FormGroup({
      username: new FormControl('', { validators: Validators.required, nonNullable: true }),
      password: new FormControl('', { validators: Validators.required, nonNullable: true }),
      otp: new FormControl(0),
    });
  }

  ngOnInit() {
    this.returnUrl = this.route.snapshot.queryParams['returnUrl'] || '/';
  }

  ngAfterViewInit() {
    setTimeout(() => {
      if (this.nameElement) {
        this.nameElement.nativeElement.focus();
      }
    }, 0);
  }

  login(): void {
    const formModel = this.form.value;
    const username = formModel.username ?? '';
    const password = formModel.password ?? '';
    const otp = formModel.otp ?? 0;
    this.auth
      .login(username, password, otp)
      // .pipe(first())
      .subscribe(
        () => {
          this.router.navigateByUrl(this.returnUrl);
        },
        (error) => {
          if (error.status === 401 && error.error.detail === 'Client is not registered') {
            this.showOtp = true;
            this.clientId = this.cs.getCookie('client_id');
          }
          this.toaster.show('Danger', error.error.details);
        },
      );
  }
}
