import { HttpClient, HttpParams } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/internal/Observable';
import { catchError } from 'rxjs/operators';

import { ErrorLoggerService } from '../core/error-logger.service';

import { Ledger } from './ledger';

const url = '/api/ledger';
const serviceName = 'LedgerService';

@Injectable({
  providedIn: 'root',
})
export class LedgerService {
  constructor(
    private http: HttpClient,
    private log: ErrorLoggerService,
  ) {}

  list(id: string | null, startDate: string | null, finishDate: string | null): Observable<Ledger> {
    const listUrl = id === null ? url : `${url}/${id}`;
    const options = { params: new HttpParams() };
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return this.http
      .get<Ledger>(listUrl, options)
      .pipe(catchError(this.log.handleError(serviceName, 'list'))) as Observable<Ledger>;
  }
}
