import { Component, OnInit } from '@angular/core';
import { FormControl, FormGroup } from '@angular/forms';
import { MatDialog } from '@angular/material/dialog';
import { ActivatedRoute, Router } from '@angular/router';
import * as moment from 'moment';

import { ToasterService } from '../../core/toaster.service';
import { ConfirmDialogComponent } from '../../shared/confirm-dialog/confirm-dialog.component';
import { Period } from '../period';
import { PeriodService } from '../period.service';

@Component({
  selector: 'app-period-detail',
  templateUrl: './period-detail.component.html',
  styleUrls: ['./period-detail.component.css'],
})
export class PeriodDetailComponent implements OnInit {
  form: FormGroup<{
    validFrom: FormControl<Date>;
    validTill: FormControl<Date>;
  }>;

  item: Period = new Period();

  constructor(
    private route: ActivatedRoute,
    private router: Router,
    private dialog: MatDialog,
    private toaster: ToasterService,
    private ser: PeriodService,
  ) {
    this.form = new FormGroup({
      validFrom: new FormControl(new Date(), { nonNullable: true }),
      validTill: new FormControl(new Date(), { nonNullable: true }),
    });
  }

  ngOnInit() {
    this.route.data.subscribe((value) => {
      const data = value as { item: Period };

      this.showItem(data.item);
    });
  }

  showItem(item: Period) {
    this.item = item;
    this.form.setValue({
      validFrom: moment(this.item.validFrom, 'DD-MMM-YYYY').toDate(),
      validTill: moment(this.item.validTill, 'DD-MMM-YYYY').toDate(),
    });
  }

  save() {
    this.ser.saveOrUpdate(this.getItem()).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/periods');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }

  delete() {
    this.ser.delete(this.item.id as string).subscribe(
      () => {
        this.toaster.show('Success', '');
        this.router.navigateByUrl('/periods');
      },
      (error) => {
        this.toaster.show('Danger', error);
      },
    );
  }

  confirmDelete(): void {
    const dialogRef = this.dialog.open(ConfirmDialogComponent, {
      width: '250px',
      data: { title: 'Delete Period?', content: 'Are you sure? This cannot be undone.' },
    });

    dialogRef.afterClosed().subscribe((result: boolean) => {
      if (result) {
        this.delete();
      }
    });
  }

  getItem(): Period {
    const formValue = this.form.value;
    this.item.validFrom = moment(formValue.validFrom).format('DD-MMM-YYYY');
    this.item.validTill = moment(formValue.validTill).format('DD-MMM-YYYY');
    return this.item;
  }
}
